Module messages

Module messages 

Messages which are built-in for ractor’s processing routines

Additionally contains definitions for BoxedState which are used to handle strongly-typed states in a generic way without having to know the strong type in the underlying framework

Structs

BoxedState
A “boxed” message denoting a strong-type message but generic so it can be passed around without type constraints

Enums

Signal
A signal message which takes priority above all else
StopMessage
Messages to stop an actor
SupervisionEvent
A supervision event from the supervision tree