Enum ractor::actor_id::ActorId

source ·
pub enum ActorId {
    Local(u64),
    Remote(u64u64),
}
An actor’s globally unique identifier

Variants



Local(u64)

A local pid



Remote(u64u64)

A remote actor on another system (system, id)

Implementations

Determine if this actor id is a local or remote actor

Returns true if it is a local actor, false otherwise

Retrieve the PID of the actor, ignoring local/remote properties