Module port

Module port 

Port implementations for signaling and reception of messages in the Ractor environment

Most of the ports we utilize are direct aliases of tokio’s channels (in the sync feature of the crate), however there are some helpful wrappers and utilities to make working with mailbox processing in ractor easier in the actor framework.

Re-exports

pub use output::*;

Modules

output
Output ports for publish-subscribe notifications between actors

Structs

RpcReplyPort
A remote procedure call’s reply port. Wrapper of concurrency::OneshotSender with a consistent error type