Factory worker properties
Structs
- Dead
Mans Switch Configuration - The configuration for the dead-man’s switch functionality
- Dead
Mans Switch Configuration Builder - Use builder syntax to set the inputs and finish with
build(). - Worker
Properties - Properties of a worker
- Worker
Start Context - Startup context data (
Arguments) which are passed to a worker on start
Enums
- Worker
Message - Message to a worker
Traits
- Worker
- A factory worker trait, which is a basic wrapper around actor logic, with predefined type information specific to workers
- Worker
Builder - The super::Factory is responsible for spawning workers and re-spawning workers under failure scenarios. This means that it needs to understand how to build workers. The WorkerBuilder trait is used by the factory to construct new workers when needed.
- Worker
Capacity Controller - Controls the size of the worker pool by dynamically growing/shrinking the pool to requested size